@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 import abc
+import base64
+import binascii
 import configparser
 import glob
 import http.cookiejar
@@ -52,6 +54,17 @@ class ApiAuthConf(YamlSerializable):
         return ApiAuthConf.factory(get_secret(region_name=region_name,
                                               secret_name=secret_name))
 
+    @classmethod
+    def load_from_base64(cls, encoded: str):
+        try:
+            log.info("Loading API conf")
+            conf_dict = json.loads(base64.b64decode(encoded).decode('utf-8'))
+            return cls.factory(conf_dict)
+        except binascii.Error as e:
+            log.error('Error decoding API configuration. Verify it is valid base64.')
+            log.error(f'Error message: {e}')
+            raise e
+
     @staticmethod
     def factory(conf: dict) -> ApiAuthConf:
         ex: Exception
