@@ -6,20 +6,24 @@ from typing import List, Dict, Tuple, Union
 from bigeye_sdk.bigconfig_validation.big_config_reports import raise_files_contain_error_exception, MetricSuiteReport, \
     process_reports, ProcessStage
 from bigeye_sdk.bigconfig_validation.validation_context import process_validation_errors, \
-    get_validation_error_cnt, get_all_validation_erros_flat
+    get_validation_error_cnt, get_all_validation_errors_flat
 from bigeye_sdk.bigconfig_validation.yaml_validation_error_messages import SRC_NOT_EXISTS_FOR_DEPLOYMENT_ERRMSG, \
-    METRIC_APPLICATION_ERROR
+    METRIC_APPLICATION_ERROR, MISMATCHED_ATTRIBUTE_ACROSS_MULTIPLE_FILES
 from bigeye_sdk.client.datawatch_client import DatawatchClient
 from bigeye_sdk.exceptions.exceptions import FileLoadException, BigConfigValidationException, NoSourcesFoundException
 from bigeye_sdk.functions.metric_functions import _is_table_level_metric
 from bigeye_sdk.functions.search_and_match_functions import wildcard_search
 from bigeye_sdk.generated.com.bigeye.models.generated import Source, CohortDefinition, \
-    MetricSuite, CohortAndMetricDefinition, CatalogEntityType, MetricDefinition, FieldType
-from bigeye_sdk.model.big_config import BigConfig, RowCreationTimes, TagDeployment, TableDeployment
-from bigeye_sdk.model.protobuf_enum_facade import SimplePredefinedMetricName
-from bigeye_sdk.model.protobuf_message_facade import SimpleCollection, SimpleMetricDefinition, SimpleSLA
+    MetricSuite, CohortAndMetricDefinition, CatalogEntityType, MetricDefinition, FieldType, NamedSchedule
+from bigeye_sdk.log import get_logger
+from bigeye_sdk.model.big_config import BigConfig, RowCreationTimes, TagDeployment, TableDeployment, \
+    SavedMetricDefinitions
+from bigeye_sdk.model.protobuf_enum_facade import SimplePredefinedMetricName, SimpleLookbackType
+from bigeye_sdk.model.protobuf_message_facade import SimpleCollection, SimpleMetricDefinition, SimpleSLA, SimpleLookback
 from bigeye_sdk.serializable import File, BIGCONFIG_FILE
 
+log = get_logger(__file__)
+
 
 def get_fq_name_from_cohort(cohort: CohortDefinition, source_name: str = None):
     """
@@ -40,23 +44,27 @@ def get_fq_name_from_cohort(cohort: CohortDefinition, source_name: str = None):
         return f'{source_name}.{r}'
 
 
-def _find_bigconfig_files(source_path: str = None) -> List[BIGCONFIG_FILE]:
+def _find_bigconfig_files(source_paths: List[str] = [], recursive: bool = False) -> List[BIGCONFIG_FILE]:
     """
     Finds bigconfig files either in specified source path or in working directory
     Args:
-        source_path: specify a source path or working directory will be used.
+        source_paths: specify List of source paths or working directory will be used.
 
     Returns: None
 
     """
-    files: List[Path]
-    if source_path and os.path.isfile(source_path):
-        files = [Path(source_path)]
-    elif source_path and os.path.isdir(source_path):
-        files = list(Path(source_path).glob('*.y*ml'))
-    else:
+    files: List[Path] = []
+    for sp in source_paths:
+        if os.path.isfile(sp):
+            files.append(Path(sp))
+        elif recursive:
+            files.extend(list(Path(sp).rglob('*.y*ml')))
+        else:
+            files.extend(list(Path(sp).glob('*.y*ml')))
+
+    if not files:
         """Assumes driver run in directory containing Bigconfig YAML."""
-        files = list(Path.cwd().glob('*.y*ml'))
+        files = list(Path.cwd().rglob('*.y*ml')) if recursive else list(Path.cwd().glob('*.y*ml'))
 
     bigeye_files: List[BIGCONFIG_FILE] = []
     file: BIGCONFIG_FILE
@@ -64,14 +72,11 @@ def _find_bigconfig_files(source_path: str = None) -> List[BIGCONFIG_FILE]:
         try:
             """Loading Bigconfig YAML.  If file is not of Bigconfig type then the error will be caught and passed."""
             bigeye_files.append(File.load(str(file)))
-        except FileLoadException:
-            pass
+        except FileLoadException as e:
+            log.warning(f'File {file} failed to load with error: {str(e.message)}')
 
     if len(bigeye_files) == 0:
-        raise BigConfigValidationException(f'No conforming files found.')
-    elif len(bigeye_files) > 1 or bigeye_files[0].type != 'BIGCONFIG_FILE':
-        # TODO change once we allow additional files.
-        raise BigConfigValidationException(f'Multiple files not currently supported.')
+        raise BigConfigValidationException(f'No conforming files found. There may be a formatting issue with the yaml.')
 
     else:
         return bigeye_files
@@ -90,10 +95,10 @@ class MetricSuiteController:
             for pmn in self.client.get_table_level_metrics().metric_names
         ]
 
-    def _upsert_slas(self,
-                     bigconfig: BigConfig,
-                     overwrite: bool,
-                     apply=False) -> List[Union[SimpleSLA, SimpleCollection]]:
+    def _upsert_collections(self,
+                            bigconfig: BigConfig,
+                            overwrite: bool,
+                            apply=False) -> List[Union[SimpleSLA, SimpleCollection]]:
         """
         Currently operates as an overwrite of existing metrics.  This prevents altering in the front end.  Every time the
         config is run it will overwrite what is already there.  Does not add metrics.
@@ -103,28 +108,27 @@ class MetricSuiteController:
         Returns: a list of upserted SLAs
 
         """
-        existing_slas = {sla.name: sla for sla in self.client.get_collections().collections}
-        deployment_slas = []
-
-        for sla in bigconfig.get_collections():
-            if sla.name in existing_slas.keys():
-                existing = SimpleCollection.from_datawatch_object(existing_slas[sla.name])
-                merged = sla.merge_for_upsert(existing=existing, overwrite=overwrite)
-                deployment_slas.append(merged)
-                if apply:
+        existing_collections = {c.name: c for c in self.client.get_collections().collections}
+        deployment_collections = []
+        if apply:
+            for collection in bigconfig.get_collections():
+                if collection.name in existing_collections.keys():
+                    existing = SimpleCollection.from_datawatch_object(existing_collections[collection.name])
+                    merged = collection.merge_for_upsert(existing=existing, overwrite=overwrite)
+                    deployment_collections.append(merged)
                     self.client.update_collection(collection=merged.to_datawatch_object())
-            else:
-                c = sla.to_datawatch_object()
-                c = self.client.create_collection(
-                    collection_name=c.name,
-                    description=c.description,
-                    metric_ids=c.metric_ids,
-                    notification_channels=c.notification_channels,
-                    muted_until_timestamp=c.muted_until_timestamp
-                ).collection
-                deployment_slas.append(SimpleCollection.from_datawatch_object(c))
+                else:
+                    c = collection.to_datawatch_object()
+                    c = self.client.create_collection(
+                        collection_name=c.name,
+                        description=c.description,
+                        metric_ids=c.metric_ids,
+                        notification_channels=c.notification_channels,
+                        muted_until_timestamp=c.muted_until_timestamp
+                    ).collection
+                    deployment_collections.append(SimpleCollection.from_datawatch_object(c))
 
-        return deployment_slas
+        return deployment_collections
 
     def row_creation_times_to_cohort(self, row_creation_times: RowCreationTimes) -> Dict[int, List[CohortDefinition]]:
         r: Dict[int, List[CohortDefinition]] = {}
@@ -150,7 +154,7 @@ class MetricSuiteController:
                                       table_name_pattern=table_pattern,
                                       column_name_pattern=column_pattern,
                                       column_type=cs.type.to_datawatch_object() if cs.type
-                                                    else FieldType.FIELD_TYPE_UNSPECIFIED,
+                                      else FieldType.FIELD_TYPE_UNSPECIFIED,
                                       entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_FIELD)
                 if id in r.keys():
                     r[id].append(cd)
@@ -193,13 +197,18 @@ class MetricSuiteController:
 
     def tag_deployment_to_cohort_and_metric_def(self,
                                                 tag_deployment: TagDeployment,
-                                                deployment_sla: Union[SimpleSLA, SimpleCollection] = None
+                                                named_schedules: List[NamedSchedule],
+                                                default_lookback: SimpleLookback,
+                                                deployment_collection: Union[SimpleSLA, SimpleCollection] = None
                                                 ) -> Dict[int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TagDeployment object.   For table level metrics
         Args:
+
             tag_deployment: tag deployment to convert to cohort and metric definition.
-            deployment_sla: The sla associated with the deployment.
+            named_schedules: a list of named schedules from the workspace
+            default_lookback: the default lookback corresponding to "metric.data_time_window.default"
+            deployment_collection: The collection associated with the deployment.
 
         Returns: Dict[source_id: int, List[CohortAndMetricDefinition]] matched based on source_id to consolidate to
         a single metric suite object.
@@ -211,8 +220,14 @@ class MetricSuiteController:
 
         for m in tag_deployment.metrics:
             """segregate table level metrics from column level and add slas"""
-            if deployment_sla and deployment_sla.id not in m.collection_ids:
-                m.collection_ids.append(deployment_sla.id)
+            if deployment_collection and deployment_collection.id not in m.collection_ids:
+                m.collection_ids.append(deployment_collection.id)
+            if m.metric_schedule and m.metric_schedule.named_schedule:
+                m.metric_schedule.named_schedule.id = [s.id for s in named_schedules
+                                                       if s.name == m.metric_schedule.named_schedule.name][0]
+            if not m.lookback:
+                m.lookback = default_lookback
+
             mdwo = m.to_datawatch_object()
             if _is_table_level_metric(metric_type=m.metric_type, table_level_metrics=self.table_level_metrics):
                 mdwo.is_table_metric = True
@@ -240,7 +255,7 @@ class MetricSuiteController:
                                                   table_name_pattern=table_pattern,
                                                   column_name_pattern=column_pattern,
                                                   column_type=cs.type.to_datawatch_object() if cs.type
-                                                    else FieldType.FIELD_TYPE_UNSPECIFIED,
+                                                  else FieldType.FIELD_TYPE_UNSPECIFIED,
                                                   entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_FIELD)
                 for sid in matching_source_ids:
                     """Tag deployments support source patterns.  Each source id becomes a key in the returned dictionary 
@@ -265,7 +280,7 @@ class MetricSuiteController:
                 table_cohort = CohortDefinition(schema_name_pattern=schema_pattern, table_name_pattern=table_pattern,
                                                 column_name_pattern=column_pattern,
                                                 column_type=cs.type.to_datawatch_object() if cs.type
-                                                    else FieldType.FIELD_TYPE_UNSPECIFIED,
+                                                else FieldType.FIELD_TYPE_UNSPECIFIED,
                                                 entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_DATASET)
 
                 for sid in matching_source_ids:
@@ -281,12 +296,16 @@ class MetricSuiteController:
 
     def table_deployment_to_cohort_and_metric_def(self,
                                                   table_deployment: TableDeployment,
-                                                  deployment_sla: Union[SimpleSLA, SimpleCollection] = None) -> Dict[
-        int, List[CohortAndMetricDefinition]]:
+                                                  named_schedules: List[NamedSchedule],
+                                                  default_lookback: SimpleLookback,
+                                                  deployment_sla: Union[SimpleSLA, SimpleCollection] = None) -> \
+            Dict[int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TableDeployment object
         Args:
             table_deployment: table deployment from which to generate cohort and metric definition
+            named_schedules: a list of named schedules from the workspace
+            default_lookback: the lookback corresponding to "metric.data_time_window.default"
             deployment_sla: SLA to which metrics will be added.
 
         Returns: Dict[warehouse_id: int, CohortAndMetricDefinition]
@@ -330,6 +349,11 @@ class MetricSuiteController:
             else:
                 if deployment_sla and deployment_sla.id not in m.sla_ids:
                     m.sla_ids.append(deployment_sla.id)
+                if m.metric_schedule and m.metric_schedule.named_schedule:
+                    m.metric_schedule.named_schedule.id = [s.id for s in named_schedules
+                                                           if s.name == m.metric_schedule.named_schedule.name][0]
+                if not m.lookback:
+                    m.lookback = default_lookback
                 mdwo = m.to_datawatch_object()
                 mdwo.is_table_metric = True
                 table_metrics.append(mdwo)
@@ -362,6 +386,11 @@ class MetricSuiteController:
                 else:
                     if deployment_sla and deployment_sla.id not in m.collection_ids:
                         m.collection_ids.append(deployment_sla.id)
+                    if m.metric_schedule and m.metric_schedule.named_schedule:
+                        m.metric_schedule.named_schedule.id = [s.id for s in named_schedules
+                                                               if s.name == m.metric_schedule.named_schedule.name][0]
+                    if not m.lookback:
+                        m.lookback = default_lookback
                     mdwo = m.to_datawatch_object()
                     col_metrics.append(mdwo)
 
@@ -376,12 +405,13 @@ class MetricSuiteController:
 
     def bigconfig_to_metric_suites(self,
                                    bigconfig: BigConfig,
-                                   deployment_slas: Dict[str, Union[SimpleSLA, SimpleCollection]]) -> List[MetricSuite]:
+                                   deployment_collections: Dict[str, Union[SimpleSLA, SimpleCollection]]) -> \
+            List[MetricSuite]:
         """
         Creates a MetricSuite for each source identified in a Bigconfig Table or Tag Deployment.
         Args:
             bigconfig: Bigconfig from which MetricSuites will be created.
-            deployment_slas: Upserted SLAs (must contain ID)
+            deployment_collections: Upserted SLAs (must contain ID)
 
         Returns: List[MetricSuite]
         """
@@ -392,17 +422,25 @@ class MetricSuiteController:
         cmds: Dict[int, List[CohortAndMetricDefinition]] = {}
 
         rct_cohorts: Dict[int, List[CohortDefinition]] = self.row_creation_times_to_cohort(bigconfig.row_creation_times)
+        named_schedules: List[NamedSchedule] = self.client.get_named_schedule().named_schedules
+        default_lookback_type: bool = [ac.boolean_value for ac in self.client.get_advanced_configs()
+                                       if ac.key == "metric.data_time_window.default"][0]
+
+        default_lookback = SimpleLookback() if default_lookback_type \
+            else SimpleLookback(lookback_type=SimpleLookbackType.DATA_TIME)
 
         for tag_d_suite in bigconfig.tag_deployments:
             for tag_d in tag_d_suite.deployments:
                 """Process all tag deployments into CohortAndMetricDefinitions.  One tag deployment per CMD"""
-                if tag_d_suite.sla:
-                    deployment_sla = deployment_slas.get(tag_d_suite.sla.name, None)
+                if tag_d_suite.collection:
+                    deployment_sla = deployment_collections.get(tag_d_suite.collection.name, None)
                 else:
                     deployment_sla = None
                 r: Dict[int, List[CohortAndMetricDefinition]] = self.tag_deployment_to_cohort_and_metric_def(
                     tag_deployment=tag_d,
-                    deployment_sla=deployment_sla
+                    named_schedules=named_schedules,
+                    default_lookback=default_lookback,
+                    deployment_collection=deployment_sla
                 )
                 for sid, definitions in r.items():
                     """consolidate based on source"""
@@ -414,12 +452,14 @@ class MetricSuiteController:
         for table_d_suite in bigconfig.table_deployments:
             for table_d in table_d_suite.deployments:
                 "Process all table deployments into CohortAndMetricDefinitions. One table deployment per CMD"
-                if table_d_suite.sla:
-                    deployment_sla = deployment_slas.get(table_d_suite.sla.name, None)
+                if table_d_suite.collection:
+                    deployment_sla = deployment_collections.get(table_d_suite.collection.name, None)
                 else:
                     deployment_sla = None
                 r = self.table_deployment_to_cohort_and_metric_def(
                     table_deployment=table_d,
+                    named_schedules=named_schedules,
+                    default_lookback=default_lookback,
                     deployment_sla=deployment_sla
                 )
                 for sid, definitions in r.items():
@@ -436,7 +476,8 @@ class MetricSuiteController:
                         rct_cohorts[sid] = [cohorts]
 
         deployment_metric_suites: Dict[int, MetricSuite] = {
-            source_id: MetricSuite(source_id=source_id, definitions=definitions, auto_apply_on_indexing=bigconfig.auto_apply_on_indexing)
+            source_id: MetricSuite(source_id=source_id, definitions=definitions,
+                                   auto_apply_on_indexing=bigconfig.auto_apply_on_indexing)
             for source_id, definitions in cmds.items()
         }
 
@@ -449,9 +490,9 @@ class MetricSuiteController:
             metric_suites.append(metric_suite)
 
         for source_id, rcts in rct_cohorts.items():
-            #TODO: Will need to refactor when supporting multiple files
             metric_suites.append(
-                MetricSuite(source_id=source_id, row_creation_cohorts=rcts, auto_apply_on_indexing=bigconfig.auto_apply_on_indexing)
+                MetricSuite(source_id=source_id, row_creation_cohorts=rcts,
+                            auto_apply_on_indexing=bigconfig.auto_apply_on_indexing)
             )
 
         return metric_suites
@@ -477,54 +518,100 @@ class MetricSuiteController:
             sys.exit(e.message)
 
     def execute_bigconfig(self,
-                          input_path: str = Path.cwd(),
-                          output_path: str = Path.cwd(), apply: bool = False):
+                          input_path: List[str] = (Path.cwd()),
+                          output_path: str = Path.cwd(),
+                          apply: bool = False,
+                          no_queue: bool = False, recursive: bool = False):
         """
         Executes an Apply or Plan for a Big Config.
         Args:
             input_path: path of source files.  If no path is given the current working directory will be used.
             output_path: path to dump the reports.  If no path is given the current working directory will be used.
             apply: If true then Big Config will be applied to the workspace.  If false then a plan will be generated.
-
+            no_queue: If true, the bigconfig will be run without queuing
+            recursive: If true, search for files recursively
         Returns: None
 
         """
-        files: List[BIGCONFIG_FILE] = _find_bigconfig_files(input_path)
+        files: List[BIGCONFIG_FILE] = _find_bigconfig_files(input_path, recursive=recursive)
 
-        bigconfig: BigConfig = files[0]
+        bigconfig: BigConfig = combine_bigconfigs(files)
 
         if get_validation_error_cnt():
             """Processing validation errors if any exist and throw exception."""
             fixme_file_list = process_validation_errors(output_path)
-            unmatched_validations_errors = get_all_validation_erros_flat(only_unmatched=True)
+            unmatched_validations_errors = get_all_validation_errors_flat(only_unmatched=True)
             raise_files_contain_error_exception(validation_error_cnt=get_validation_error_cnt(),
                                                 unmatched_validations_errors=unmatched_validations_errors,
                                                 fixme_file_list=fixme_file_list)
 
         # Creates new SLAs only so that all ids exist when we create the metric suites.
-        deployment_slas = {sla.name: sla for sla in self._upsert_slas(bigconfig=bigconfig, overwrite=False,
-                                                                      apply=False)}
+        deployment_collections = {c.name: c for c in self._upsert_collections(bigconfig=bigconfig, overwrite=False,
+                                                                              apply=apply)}
 
-        metric_suites = self.bigconfig_to_metric_suites(bigconfig=bigconfig, deployment_slas=deployment_slas)
+        metric_suites = self.bigconfig_to_metric_suites(bigconfig=bigconfig,
+                                                        deployment_collections=deployment_collections)
 
         if get_validation_error_cnt():
             """Processing validation errors if any exist and throw exception."""
             fixme_file_list = process_validation_errors(output_path)
-            unmatched_validations_errors = get_all_validation_erros_flat(only_unmatched=True)
+            unmatched_validations_errors = get_all_validation_errors_flat(only_unmatched=True)
             raise_files_contain_error_exception(validation_error_cnt=get_validation_error_cnt(),
                                                 unmatched_validations_errors=unmatched_validations_errors,
                                                 fixme_file_list=fixme_file_list)
 
         # Applies changes and overwrites once local validations have passed.
         if apply:
-            self._upsert_slas(bigconfig=bigconfig, overwrite=True, apply=apply)  # Will always overwrite for Bigconfig.
+            self._upsert_collections(bigconfig=bigconfig, overwrite=True,
+                                     apply=apply)  # Will always overwrite for Bigconfig.
 
         for metric_suite in metric_suites:
             j = metric_suite.to_json()
-            response = self.client.post_metric_suite(metric_suite=metric_suite, apply=apply)
+            if not apply or no_queue:
+                response = self.client.post_metric_suite(metric_suite=metric_suite, apply=apply)
+            else:
+                response = self.client.post_metric_suite_queue(metric_suite=metric_suite)
             process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
             MetricSuiteReport.from_datawatch_object(response,
                                                     source_name=self.sources_by_id_ix[metric_suite.source_id].name,
                                                     process_stage=process_stage)
 
         process_reports(output_path=output_path)
+
+
+def combine_bigconfigs(bigconfigs: List[BigConfig]) -> BigConfig:
+    to_return: BigConfig = BigConfig(
+        type="BIGCONFIG_FILE",
+        tag_definitions=[],
+        row_creation_times=RowCreationTimes(),
+        saved_metric_definitions=SavedMetricDefinitions(metrics=[]),
+        tag_deployments=[],
+        table_deployments=[]
+    )
+    auto_apply: bool = bigconfigs[-1].auto_apply_on_indexing
+    for bc in bigconfigs:
+        if auto_apply != bc.auto_apply_on_indexing:
+            "registering validation errors when auto_apply doesn't match across bigconfigs"
+            errlns = [f"auto_apply_on_indexing: {auto_apply}\n",
+                      f"auto_apply_on_indexing: {bc.auto_apply_on_indexing}\n"]
+            BigConfig.register_validation_error(error_lines=errlns,
+                                                error_message=MISMATCHED_ATTRIBUTE_ACROSS_MULTIPLE_FILES.format(
+                                                    attribute="auto_apply_on_indexing")
+                                                )
+        to_return.auto_apply_on_indexing = auto_apply
+        to_return.tag_definitions.extend(bc.tag_definitions)
+        if bc.row_creation_times:
+            rct = bc.row_creation_times
+            to_return.row_creation_times.column_selectors.extend(
+                rct.column_selectors)
+            to_return.row_creation_times.tag_ids.extend(rct.tag_ids)
+        if bc.saved_metric_definitions:
+            smd = bc.saved_metric_definitions
+            to_return.saved_metric_definitions.metrics.extend(smd.metrics)
+        to_return.tag_deployments.extend(bc.tag_deployments)
+        to_return.table_deployments.extend(bc.table_deployments)
+
+    to_return.build_tag_ix(to_return.tag_definitions)
+    to_return.build_saved_metric_ix(to_return.saved_metric_definitions)
+
+    return to_return
