@@ -7,6 +7,7 @@ class AttributeTypeException(Exception):
     def __init__(self, message: str):
         self.message = message
 
+
 class InvalidConfigurationException(Exception):
     def __init__(self, message: str):
         self.message = message
@@ -25,3 +26,13 @@ class BigConfigValidationException(Exception):
 class NoSourcesFoundException(Exception):
     def __init__(self, message: str):
         self.message = message
+
+
+class TableNotFoundException(Exception):
+    def __init__(self, message: str):
+        self.message = message
+
+
+class BigconfigIncompleteException(Exception):
+    def __init__(self, message: str):
+        self.message = message
