@@ -2,7 +2,6 @@ from __future__ import annotations
 
 from typing import List, Optional, Dict, Tuple, Any, Union
 
-
 import yaml
 from pydantic import Field, PrivateAttr, validator, root_validator
 
@@ -48,8 +47,6 @@ class ColumnSelector(YamlModelWithValidatorContext):
 
         return values
 
-
-
     def __hash__(self):
         return hash(self.name)
 
@@ -91,6 +88,7 @@ class TagDefinition(YamlModelWithValidatorContext):
         must_be_list_validator(clazz=TagDefinition, attribute_name='column_selectors', values=values)
 
         return values
+
     def __hash__(self):
         return hash((repr(self.tag_id), self.column_selectors))
 
@@ -111,6 +109,7 @@ class SavedMetricDefinitions(YamlModelWithValidatorContext):
 
         return values
 
+
 class RowCreationTimes(YamlModelWithValidatorContext):
     tag_ids: Optional[List[str]] = Field(default_factory=lambda: [])
     column_selectors: Optional[List[ColumnSelector]] = Field(default_factory=lambda: [])
@@ -135,7 +134,6 @@ class TagDeployment(YamlModelWithValidatorContext):
 
         return values
 
-
     def explode_fq_column_selectors(self) -> List[List[str]]:
         """
         Explodes a fully qualified column selectors into a list of names.  Supports wild cards as *. Supports single and
@@ -186,6 +184,7 @@ class ColumnMetricDeployment(YamlModelWithValidatorContext):
 
         return values
 
+
 class TableDeployment(YamlModelWithValidatorContext):
     fq_table_name: str
     columns: Optional[List[ColumnMetricDeployment]] = Field(default_factory=lambda: [])
@@ -287,15 +286,17 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
     _tag_ix_: Dict[str, List[ColumnSelector]] = PrivateAttr({})  # Dict[tag_id, List[ColumnSelector]]
     _saved_metric_ix_: Dict[str, SimpleMetricDefinition] = PrivateAttr(
         {})  # Dict[saved_metric_id, SimpleMetricDefinition]
+    _raw_: dict = PrivateAttr({})
 
     # _saved_metric_collection_ix_: Dict[str, List, str] = {}  # Dict[saved_metric_collection_id, List[saved_metric_id]] (V1)
 
     @root_validator(pre=True)
-    def must_be_list(cls, values):
+    def bigconfig_validation_checks(cls, values):
         must_be_list_validator(clazz=BigConfig, attribute_name='tag_definitions', values=values)
         must_be_list_validator(clazz=BigConfig, attribute_name='tag_deployments', values=values)
         must_be_list_validator(clazz=BigConfig, attribute_name='table_deployments', values=values)
 
+        values['_raw_'] = values
         return values
 
     @validator('saved_metric_definitions')
@@ -311,23 +312,29 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
         super().__init__(**data)
 
         log.info('Building Indices.')
-        if self.tag_definitions:
-            self._tag_ix_ = self._generate_tag_ix(self.tag_definitions)
+        self.build_tag_ix(self.tag_definitions)
 
-        if self.saved_metric_definitions:
+        self.build_saved_metric_ix(self.saved_metric_definitions)
+
+    def build_tag_ix(self, tag_definitions: List[TagDefinition]):
+        if tag_definitions:
+            self._tag_ix_ = self._generate_tag_ix(tag_definitions)
+
+    def build_saved_metric_ix(self, saved_metric_definitions: SavedMetricDefinitions):
+        if saved_metric_definitions:
             self._saved_metric_ix_ = self._generate_saved_metric_def_ix(self.saved_metric_definitions)
 
     def apply_tags_and_saved_metrics(self):
 
         log.info('Applying tags and saved metrics.')
 
-        if self._tag_ix_:
+        if self._tag_ix_ or self.tag_deployments:
             apply_result = self._apply_tags(tag_ix=self._tag_ix_, tag_deps=self.tag_deployments,
                                             row_creation_times=self.row_creation_times)
             self.tag_deployments = apply_result[0]
             self.row_creation_times = apply_result[1]
 
-        if self._saved_metric_ix_:
+        if self._saved_metric_ix_ or self.tag_deployments or self.table_deployments:
             apply_result = self._apply_saved_metrics(saved_metric_ix=self._saved_metric_ix_,
                                                      tag_deps=self.tag_deployments,
                                                      table_deps=self.table_deployments)
@@ -389,6 +396,16 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
 
         return smdix
 
+    @classmethod
+    def _saved_metric_id_exists_in_ix(cls, smd_id: str, saved_metric_ix: Dict[str, SimpleMetricDefinition]) -> bool:
+        if smd_id not in saved_metric_ix.keys():
+            error_message = SAVED_METRIC_ID_NOT_EXISTS_IN_SAVED_METRICS_DEFINITION_ERRMSG.format(saved_metric_id=smd_id)
+            cls.register_validation_error(error_lines=[smd_id],
+                                          error_message=error_message)
+            return False
+        else:
+            return True
+
     @classmethod
     def _tag_id_exists_in_ix(cls, tag_id: str, tag_ix: Dict[str, List[ColumnSelector]]) -> bool:
         if tag_id not in tag_ix:
