@@ -14,7 +14,7 @@ from bigeye_sdk.bigconfig_validation.validation_functions import safe_split_dict
 from bigeye_sdk.bigconfig_validation.yaml_model_base import YamlModelWithValidatorContext
 from bigeye_sdk.bigconfig_validation.yaml_validation_error_messages import FORMATTING_ERRMSG, \
     OVERRIDE_METRIC_TYPE_SAVED_METRIC_ERRMSG, MUST_HAVE_METRIC_TYPE_ERRMSG, MUST_HAVE_METRIC_ID_ERRMSG, \
-    METRIC_TYPE_NOT_EXISTS_ERRMSG, POSSIBLE_MATCH_ERRMSG, NO_POSSIBLE_MATCH_ERRMSG
+    METRIC_TYPE_NOT_EXISTS_ERRMSG, POSSIBLE_MATCH_ERRMSG, NO_POSSIBLE_MATCH_ERRMSG, TWO_SCHEDULES_DEFINED
 from bigeye_sdk.exceptions import InvalidConfigurationException
 from bigeye_sdk.functions.search_and_match_functions import fuzzy_match
 from bigeye_sdk.functions.table_functions import get_table_column_id
@@ -22,7 +22,7 @@ from bigeye_sdk.generated.com.bigeye.models.generated import Threshold, SimpleBo
     SimpleBound, NamedSchedule, ComparisonColumnMapping, Table, IdAndDisplayName, ColumnNamePair, TimeInterval, \
     MetricParameter, MetricType, PredefinedMetric, TemplateMetric, AutoThreshold, ForecastModelType, RelativeThreshold, \
     StandardDeviationThreshold, FreshnessScheduleThreshold, NotificationChannel, MetricDefinition, MetricConfiguration, \
-    Collection, SlackChannelInfo
+    Collection, SlackChannelInfo, Webhook, WebhookHeader, MetricSchedule, TimeIntervalType
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.base_datawatch_facade import DatawatchFacade
 from bigeye_sdk.model.protobuf_enum_facade import SimplePredefinedMetricName, SimpleTimeIntervalType, \
@@ -49,6 +49,10 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
                                   SimplePredefinedMetricName.FRESHNESS]
         return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in freshness_metric_names]
 
+    @classmethod
+    def is_freshness_metric(cls, predefined_metric):
+        return predefined_metric in cls.get_freshness_metric_types()
+
     @classmethod
     def from_datawatch_object(cls, obj: MetricType) -> SimpleMetricType:
         t = betterproto.which_one_of(obj, "metric_type")[0]
@@ -120,8 +124,8 @@ class SimpleTemplateMetric(SimpleMetricType, type='TEMPLATE'):
 
 
 class SimpleNamedSchedule(YamlModelWithValidatorContext, DatawatchFacade):
-    name: str
-    cron: str
+    name: str = None
+    cron: Optional[str] = None
     id: Optional[int] = None
 
     @classmethod
@@ -135,7 +139,7 @@ class SimpleNamedSchedule(YamlModelWithValidatorContext, DatawatchFacade):
 class SimpleColumnMapping(YamlModelWithValidatorContext, DatawatchFacade):
     source_column_name: str
     target_column_name: str
-    metrics: List[SimpleMetricType]
+    metrics: List[SimplePredefinedMetric]
 
     @root_validator(pre=True)
     def must_be_list(cls, values):
@@ -190,6 +194,43 @@ class SimpleTimeInterval(YamlModelWithValidatorContext, DatawatchFacade):
         return TimeInterval(interval_type=self.interval_type.to_datawatch_object(), interval_value=self.interval_value)
 
 
+class SimpleMetricSchedule(YamlModelWithValidatorContext, DatawatchFacade):
+    schedule_frequency: Optional[SimpleTimeInterval] = None
+    named_schedule: Optional[SimpleNamedSchedule] = None
+
+    @classmethod
+    def from_datawatch_object(cls, obj: MetricSchedule) -> Optional[SimpleMetricSchedule]:
+        if obj.schedule_frequency:
+            if obj.schedule_frequency.interval_type == 0 and obj.schedule_frequency.interval_value == 0:
+                return None
+            else:
+                sti = SimpleTimeInterval(
+                    interval_type=SimpleTimeIntervalType.from_datawatch_object(
+                        obj.schedule_frequency.interval_type),
+                    interval_value=obj.schedule_frequency.interval_value
+                )
+                return SimpleMetricSchedule(schedule_frequency=sti)
+        else:
+            return SimpleMetricSchedule(
+                named_schedule=SimpleNamedSchedule.from_datawatch_object(obj.named_schedule)
+            )
+
+    def to_datawatch_object(self) -> MetricSchedule:
+        ms = MetricSchedule()
+        if self.named_schedule:
+            ms.named_schedule = NamedSchedule(
+                id=self.named_schedule.id,
+                name=self.named_schedule.name,
+                cron=self.named_schedule.cron
+            )
+        elif self.schedule_frequency:
+            ms.schedule_frequency = TimeInterval(
+                interval_type=self.schedule_frequency.interval_type.to_datawatch_object(),
+                interval_value=self.schedule_frequency.interval_value
+            )
+        return ms
+
+
 ST = TypeVar('ST', bound='SimpleThreshold')
 
 
@@ -392,6 +433,15 @@ class SimpleNotificationChannel(PydanticSubtypeSerializable, DatawatchFacade, AB
                 slack=obj.slack_channel_info.channel_name,
                 thread_ts=obj.slack_channel_info.thread_ts
             )
+        elif t == 'webhook':
+            key: str
+            value: str
+            if obj.webhook.webhook_headers:
+                key = obj.webhook.webhook_headers[0].key
+                value = obj.webhook.webhook_headers[0].value
+            return WebhookNotificationChannel(webhook=obj.webhook.webhook_url,
+                                              webhook_header_key=key,
+                                              webhook_header_value=value)
         else:
             error_message = f"Notification channel type not supported: {t}"
             raise InvalidConfigurationException(error_message)
@@ -401,16 +451,6 @@ class EmailNotificationChannel(SimpleNotificationChannel, type='EMAIL'):
     type = 'EMAIL'
     email: str
 
-    @validator('email')
-    def must_have_correct_email_format(cls, v):
-        p = re.compile(r'([A-Za-z0-9]+[.-_+])*[A-Za-z0-9]+@[A-Za-z0-9-]+(\.[A-Z|a-z]{2,})+')
-        if not re.fullmatch(p, v):
-            error_message = FORMATTING_ERRMSG.format(s=v)
-            cls.register_validation_error(error_lines=[v],
-                                          error_message=error_message)
-
-        return v
-
     def to_datawatch_object(self, **kwargs) -> NotificationChannel:
         return NotificationChannel(email=self.email)
 
@@ -436,6 +476,21 @@ class SlackNotificationChannel(SimpleNotificationChannel, type='SLACK'):
         return NotificationChannel(slack_channel_info=sci)
 
 
+class WebhookNotificationChannel(SimpleNotificationChannel, type='WEBHOOK'):
+    type = 'WEBHOOK'
+    webhook: str
+    webhook_header_key: Optional[str] = None
+    webhook_header_value: Optional[str] = None
+
+    def to_datawatch_object(self, **kwargs) -> NotificationChannel:
+        if self.webhook_header_key and self.webhook_header_value:
+            header = WebhookHeader(key=self.webhook_header_key, value=self.webhook_header_value)
+            return NotificationChannel(webhook=Webhook(webhook_url=self.webhook,
+                                                       webhook_headers=[header]))
+        else:
+            return NotificationChannel(webhook=Webhook(webhook_url=self.webhook))
+
+
 class SimpleMetricParameter(YamlModelWithValidatorContext, DatawatchFacade):
     key: str
     string_value: Optional[str] = None
@@ -455,6 +510,7 @@ class SimpleMetricParameter(YamlModelWithValidatorContext, DatawatchFacade):
                                column_name=self.column_name,
                                number_value=self.number_value)
 
+
 @deprecated('SimpleSLA is deprecated and will be removed in future versions. Use SimpleCollection instead.')
 class SimpleSLA(YamlModelWithValidatorContext, DatawatchFacade):
     name: str
@@ -542,10 +598,11 @@ class SimpleSLA(YamlModelWithValidatorContext, DatawatchFacade):
         )
 
         return c
-    
+
     def to_simple_collection(self) -> SimpleCollection:
         return SimpleCollection.parse_obj(self)
 
+
 class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
     name: str
     id: int = None
@@ -617,7 +674,7 @@ class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
 
         if obj.notification_channels:
             collection.notification_channels = [SimpleNotificationChannel.from_datawatch_object(nc)
-                                         for nc in obj.notification_channels]
+                                                for nc in obj.notification_channels]
 
         return collection
 
@@ -633,6 +690,7 @@ class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
 
         return c
 
+
 class BucketSize(YamlStrEnum):
     DAY = "DAY"  # 86400
     HOUR = "HOUR"  # 3600
@@ -658,7 +716,7 @@ class BucketSize(YamlStrEnum):
 class SimpleLookback(YamlModelWithValidatorContext):
     lookback_window: Optional[SimpleTimeInterval] = SimpleTimeInterval(interval_type=SimpleTimeIntervalType.DAYS,
                                                                        interval_value=2)
-    lookback_type: Optional[SimpleLookbackType] = SimpleLookbackType.DATA_TIME
+    lookback_type: Optional[SimpleLookbackType] = SimpleLookbackType.METRIC_TIME
     bucket_size: Optional[Union[int, BucketSize]] = BucketSize.DAY
 
 
@@ -673,10 +731,11 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     threshold: Optional[SimpleThreshold] = None
     notification_channels: Optional[List[SimpleNotificationChannel]] = None
     parameters: Optional[List[SimpleMetricParameter]] = None
-    lookback: Optional[SimpleLookback] = SimpleLookback()
+    lookback: Optional[SimpleLookback] = None
     muted_until_epoch_seconds: Optional[int] = 0
     sla_ids: Optional[List[int]] = Field(default_factory=lambda: [])
     collection_ids: Optional[List[int]] = Field(default_factory=lambda: [])
+    metric_schedule: Optional[SimpleMetricSchedule] = None
 
     @validator('threshold', always=True, pre=False)
     def set_default_threshold(cls, threshold, values):
@@ -706,23 +765,33 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         return threshold
 
-    @validator('schedule_frequency', always=True, pre=False)
-    def set_default_schedule_frequency(cls, schedule_frequency, values):
+    @root_validator(pre=False)
+    def set_default_schedule_frequency(cls, values):
+        # TODO: This will change when only utilizing MetricSchedule as part of the MetricConfiguration
         metric_type = values.get('metric_type')
 
-        if schedule_frequency:
+        if values.get('schedule_frequency'):
             """For the case that a user has specifically defined a metric"""
-            return schedule_frequency
+            return values
+        elif values.get('metric_schedule'):
+            """For the case that a user has defined a metric_schedule"""
+            return values
         elif not metric_type:
             """For the case where this is a saved metric reference."""
-            return schedule_frequency
+            return values
         elif metric_type \
                 and metric_type in SimpleMetricType.get_freshness_metric_types():
             """Freshness metrics should execute every 6 hours by default WIZ-1623"""
-            return SimpleTimeInterval(interval_type=SimpleTimeIntervalType.HOURS, interval_value=6)
+            values['schedule_frequency'] = SimpleTimeInterval(
+                interval_type=SimpleTimeIntervalType.HOURS, interval_value=6
+            )
+            return values
         else:
             """All other metrics should execute every 24 hours by default WIZ-1623"""
-            return SimpleTimeInterval(interval_type=SimpleTimeIntervalType.HOURS, interval_value=24)
+            values['schedule_frequency'] = SimpleTimeInterval(
+                interval_type=SimpleTimeIntervalType.HOURS, interval_value=24
+            )
+            return values
 
     def get_error_lines(self) -> List[str]:
         """
@@ -751,7 +820,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
                     s='Metric Type is an object.')
             )
 
-            return {'type': 'PREDEFINED', 'metric': 'UNDEFINED'}
+            return {'type': 'PREDEFINED', 'predefined_metric': 'UNDEFINED'}
 
         return metric_type
 
@@ -768,11 +837,25 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     @root_validator(pre=True)
     def warn_and_use_collection(cls, values):
         if values.get('sla_ids'):
-            log.warning('sla_id class variable is deprecated and will be removed in future versions. Use collection_ids.')
+            log.warning(
+                'sla_id class variable is deprecated and will be removed in future versions. Use collection_ids.')
             values['collection_ids'] = values['sla_ids']
 
         return values
 
+    @root_validator(pre=True)
+    def schedule_check(cls, values):
+        sf = values.get('schedule_frequency')
+        ms = values.get('metric_schedule')
+        if sf and ms:
+            errlns = [yaml.safe_dump({'schedule_frequency': sf}, indent=True, sort_keys=False),
+                      yaml.safe_dump({'metric_schedule': ms}, indent=True, sort_keys=False)]
+            cls.register_validation_error(
+                error_lines=errlns,
+                error_message=TWO_SCHEDULES_DEFINED
+            )
+        return values
+
     def deployment_validations(self, config_error_lines: List[str]):
         """
         Runs all deployment validations for SimpleMetricDefinitions
@@ -853,7 +936,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         builder.muted_until_epoch_seconds = obj.muted_until_epoch_seconds
 
         builder.collection_ids = obj.collection_ids
-
+        builder.metric_schedule = SimpleMetricSchedule.from_datawatch_object(obj.metric_schedule)
         return builder
 
     def to_datawatch_object(self, **kwargs) -> MetricDefinition:
@@ -891,11 +974,17 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
             if self.lookback.lookback_type:
                 builder.lookback_type = self.lookback.lookback_type.to_datawatch_object()
 
+            """Freshness metrics require data time lookback"""
+            if SimpleMetricType.is_freshness_metric(self.metric_type):
+                builder.lookback_type = SimpleLookbackType.DATA_TIME.to_datawatch_object()
+
             builder.grain_seconds = self.lookback.bucket_size.to_seconds()
 
         builder.muted_until_epoch_seconds = self.muted_until_epoch_seconds
 
         if self.collection_ids:
             builder.collection_ids = self.collection_ids
+        if self.metric_schedule:
+            builder.metric_schedule = self.metric_schedule.to_datawatch_object()
 
         return builder
